clear all
close all

% To produce a figure showing results for a simple cell with Gaussian
% temporal kernels, using the standard disparity-averaging read-out rule:
% 'average over time and position; take centroid of resulting activity distribution';

col={'k' 'm' 'b' [0 0.75 0] [0.8 0.8 0] 'r' 'm' 'g' 'c'};

TToverSDt=[1:6];
cnt=0;
for jTT=2:length(TToverSDt)
    try
        load(['' sprintf('Fullresults_simple_gauss_theta=74deg_SDx=0.004_Tontau=%1d_jdt1_accA.mat',TToverSDt(jTT))]); 
        for jdt=1:length(dttoverT)
            try
                load(['' sprintf('Fullresults_simple_gauss_theta=74deg_SDx=0.004_Tontau=%1d_jdt%d_accA.mat',TToverSDt(jTT),jdt)]);
                T=TToverSDt(jTT)*SDt;
                X = vel .* T; % spatial period of strobe stimulus, in deg	
                dtoverT=dttoverT(jdt);
                Dt = dtoverT*T;
                dispmax = 4 * SDx + abs(vel) * (4*SDt + abs(Dt));
                dispmin = -dispmax;
                disparity = [ dispmin : (dispmax-dispmin)/(ndisparity-1) : dispmax  ] ; 
                
                dispmodterm = vL.*vR;
                disparitydistrib = squeeze(sum(sum(dispmodterm,1),2))';
                
                % average over time and position; take centroid of resulting activity distribution
                prcdisp(jdt) = sum(disparitydistrib.*disparity)./sum(disparitydistrib);
            catch
                prcdisp(jdt)=NaN;
            end % end seeing if this jdt exist
        end % next delay dt
        
        cnt=cnt+1;
        plot(dttoverT,prcdisp/X,'col',col{jTT},'linestyle','none','marker','x','linew',2);
        hold on
        finedttoverT = interp(dttoverT,10);
        h(cnt)=plot(finedttoverT,DisparityAveraging_RFconv(finedttoverT*T,T,'Gaussian',SDt),'col',col{jTT},'linestyle','-','linew',2);
        lab{cnt} = ['T/\tau' sprintf(' = %d',TToverSDt(jTT))];
        xlabel('interocular delay / interflash interval,   \Deltat/T ','fontsize',14)
        ylabel('perceived disparity / interflash distance,   \Deltax/X ','fontsize',14)
        set(gca,'fontsize',16)
        drawnow
    catch
    end % seeing if this T exists
end

set(gcf,'pos',[384   106   652   601])
axis equal
xlim([0 1]*0.5)
ylim([0 1]*0.5)
title('Gaussian temporal kernel','fontsize',18)

k=legend(h,lab);
set(k,'pos',[0.73 0.12 0.25 0.3206])

axes
delayontau = [-1:0.01:1]*5;
plot(delayontau,exp(-delayontau.^2./4),'k')
set(gca,'pos',[0.27 0.67 0.3 0.2],'box','off')
xlabel('delay / \tau','fontsize',14)
ylabel('relative weight','fontsize',14)
set(gcf,'paperpositionmode','auto')
