function MakeFigure5 % aka Fig_PopulationPlot_Tontau4B aka StrobePulfrich_disponly_WTA
clear all
close all hidden
format compact
format long
global twoSDxsq twoSDtsq twopiSF SDt temporalag
global ntime ndisparity tol
% This is based on the maths I have laid out in longversion.doc.

% This program outputs the response of a population of model neurons tuned
% to different stimulus positions and disparities, at different times
% throughout a strobe Pulfrich stimulus

% Define RF function
SDx = 0.1; % deg
SDt = 10e-3; %s
SF = 2; %cycles per deg
temporalag = 50e-3;

% Define stimulus properties
T = 4*SDt; % temporal period of strobe stimulus, in s
vel = 3.6; % speed of strobe stimulus, in deg/s
X = vel .* T; % spatial period of strobe stimulus, in deg
dtoverT = 0.4;

ndisparity = 51;
nxcen = 41;
time = [0.25 0.45 0.75 0.9]*T;
ntime = length(time);
tol = 1e-6;

twopiSF = 2*pi*SF;
twoSDxsq = 2*SDx^2;
twoSDtsq = 2*SDt^2;

Dt = dtoverT*T;
% Calculate the maximum disparity we can encounter
dispmax = 3 * X;
dispmin = -dispmax;
disparity = [ dispmin : (dispmax-dispmin)/(ndisparity-1) : dispmax  ] ; 
xmin = - 3 * X;
xmax = 3 * X;
xcen = [ xmin : (xmax-xmin)/(nxcen-1) : xmax] - vel*temporalag;
 
vLEven = zeros(nxcen,ndisparity);
vREven= vLEven;
vLOdd= vLEven;
vROdd= vLEven;
energy = zeros(ntime,nxcen,ndisparity);
summedoverxc = zeros(ndisparity,ntime);
figure('pos',[479    43   665   682],'name',sprintf('T/Sdt = %5.2f, dt/T = %5.2f',T/SDt,dtoverT),'paperpositionmode','auto')
for jt=1:ntime    
    for jx=1:nxcen

        for jd=1:ndisparity
            [vLEven(jx,jd),vREven(jx,jd),vLOdd(jx,jd),vROdd(jx,jd)] = Convolutions(time(jt),X,T,Dt,xcen(jx),disparity(jd));
        end
    end
    
    energy(jt,:,:) = (vLEven+vREven).^2; % + (vLOdd+vROdd).^2;
    dispmodterm(jt,:,:) = 2.*vLEven.*vREven; % + 2.*vLOdd.*vROdd;%
    summedoverxc(:,jt) = squeeze(sum(dispmodterm(jt,:,:),2));
    
    subplot(4,ntime,ntime+jt)
    imagesc(disparity./X,xcen./X,squeeze(energy(jt,:,:)));
    set(gca,'ydir','normal')
    drawnow
    save tmp
end

cmn = min(min(min(energy)));
cmx = max(max(max(energy)));
cmnd = min(min(min(dispmodterm)));
cmxd = max(max(max(dispmodterm)));
fs=12;
fs1=14;
fs2=12;
fs3=14;
fs5=10;
xlabpos=-0.7; % position of labels
ylabpos=-0.4;
for jt=1:ntime 
    % Draw space-time diagram
    subplot(4,ntime,jt)
    xL = [-10:10]*X;
    tL = [-10:10]*T;
    xR = [-10:10]*X;
    tR = [-10:10]*T + Dt;
    tt = [min(time)-T-SDt*4 : 0.001 : max(time)+SDt];
    tresp = RFt(tt-time(jt));
    tmp = zeros(1,length(tt),3);
    tmp(1,:,1) = 1-tresp;tmp(1,:,2) = 1-tresp;tmp(1,:,3) = 1-tresp;
    image(tt./T,xcen./X,1-tmp);
    hold on
    plot(tL/T,xL/X,'ro-','linew',2,'markerfacecol','r','markersize',3)
    plot(tR/T,xR/X,'bo-','linew',2,'markerfacecol','b','markersize',3)
    ylim([-3*SDx-abs(X) 3*SDx+abs(X)]/X)
    xlim([min(time)-T-SDt*4 max(time)+SDt]/T)
    plot([1 1]*time(jt)/T,ylim,'y','linew',2)
    title(sprintf('time = %5.2f T',time(jt)/T),'fontsize',fs1)
    set(gca,'fontsize',fs2,'ydir','normal')
    xlabel('time t/T','fontsize',fs)
    if jt==1;
        text(ylabpos,0.5,'stimulus position x/X','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs)
        text(xlabpos,0.5,'spacetime diagram','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs3)
    end
    if jt==3
        arrow([tL(10) tR(10)]/T,[xL(10) xR(10)]/X,30,0.2,'g','linew',2)
        arrow([tL(11) tR(10)]/T,[xL(11) xR(10)]/X,30,0.2,'g','linew',2)
    end
	% Mark on stimulus identity
	for j=[10 11]
		text(xL(j)/X-0.2,tL(j)/T+0.6,['L' char(39+j)],'col','r','fontsize',9,'fontw','bold')
		text(xR(j)/X+0.4,tR(j)/T-1,['R' char(39+j)],'col','b','fontsize',9,'fontw','bold')
	end
    % Draw energy
    subplot(4,ntime,ntime+ jt)
    hold on
	plot(xlim,[0 0],'k')
	plot([0 0],ylim,'k')
	caxis([cmn cmx])
    set(gca,'fontsize',fs2)
    if jt==1;
        text(ylabpos,0.5,'pref. position x_{pref}/X','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs)
        text(xlabpos,0.5,'energy unit','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs3)
    end
    % Draw disparity-modulated term
    subplot(4,ntime,ntime*2+ jt)
    imagesc(disparity./X,xcen./X,squeeze(dispmodterm(jt,:,:)));
    hold on
	plot(xlim,[0 0],'k')
	plot([0 0],ylim,'k')
    caxis([cmnd cmxd])
    set(gca,'fontsize',fs2,'ydir','normal')
    if jt==1;
        text(ylabpos,0.5,'pref. position x_{pref}/X','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs)
        text(xlabpos,0.5,'binoc. term','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs3)
    end
    % Draw disparity-modulated term summed over xcen
    subplot(4,ntime,ntime*3+ jt)
    plot(disparity./X,squeeze(sum(dispmodterm(jt,:,:),2)),'linew',1.5);
    ylim([min(min(squeeze(sum(dispmodterm,2)))) max(max(squeeze(sum(dispmodterm,2)))) ]) 
    xlim([min(disparity./X) max(disparity./X)])
    hold on
	plot(xlim,[0 0],'k')
	plot([0 0],ylim,'k')
	plot([1 1],ylim,'k:','linew',1.5)
    caxis([cmnd cmxd])
    set(gca,'fontsize',fs2,'ydir','normal','yticklabel','')
    xlabel('pref. disparity \Deltax_{pref}/X','fontsize',fs5)
    if jt==1;
        text(ylabpos,0.5,'av. activity','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs)
        text(xlabpos,0.5,'disparity distbn.','units','norm','rot',90,'vert','middle','horiz','center','fontsize',fs3)
    end
end
for j=1:ntime*4
	subplot(4,ntime,j)
	text(-0.25,1.1+0.3*(j<=4),char(64+j),'units','norm','fontsize',20,'fontweight','bold')
end
colormap hot
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [vLEven,vREven,vLOdd,vROdd] = ContinuousCase(t,X,T,Dt,xcen,disparity)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vel = X/T;
vLEven = quad(@RFLEven,-100*X,100*X,1e-7,[],xcen,disparity,vel,t);
vREven = quad(@RFREven,-100*X,100*X,1e-7,[],xcen,disparity,vel,t,Dt);
vLOdd = quad(@RFLOdd,-100*X,100*X,1e-7,[],xcen,disparity,vel,t);
vROdd = quad(@RFROdd,-100*X,100*X,1e-7,[],xcen,disparity,vel,t,Dt);


function z = RFLEven(x,xcen,Dxpref,vel,t)
z = RFxEven(x-xcen-Dxpref/2).*RFt(x/vel - t);
function z = RFREven(x,xcen,Dxpref,vel,t,Dt)
z = RFxEven(x-xcen+Dxpref/2).*RFt(x/vel + Dt - t);
function z = RFLOdd(x,xcen,Dxpref,vel,t)
z = RFxOdd(x-xcen-Dxpref/2).*RFt(x/vel - t);
function z = RFROdd(x,xcen,Dxpref,vel,t,Dt)
z = RFxOdd(x-xcen+Dxpref/2).*RFt(x/vel + Dt - t);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [vLEven,vREven,vLOdd,vROdd] = Convolutions(t,X,T,Dt,xcen,disparity)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global tol

abstol=1e-30;

jmaxL = floor( t/T);
kmaxR = floor( (t-Dt)/T);

j = jmaxL;
jsm = 0;
fracerrj = 1;
while abs(jsm)>abstol & fracerrj > tol | j>jmaxL-5 
    % To avoid making it spend ages converging accurately on a tiny value,
    % I force it to loop at least 5 times, but then stop if the answer is
    % still no larger than abstol.
    contrib = RFt(j*T-t) .* RFxEven(j*X - xcen - disparity./2)  ;
    jsm = jsm + contrib;
    j = j - 1;
    if abs(jsm)>0
        fracerrj = abs(contrib/jsm);
    end
end     
vLEven = jsm;

j = jmaxL;
jsm = 0;
fracerrj = 1;
while abs(jsm)>abstol & fracerrj > tol | j>jmaxL-5 
    % To avoid making it spend ages converging accurately on a tiny value,
    % I force it to loop at least 5 times, but then stop if the answer is
    % still no larger than abstol.
    contrib = RFt(j*T-t) .* RFxOdd(j*X - xcen - disparity./2)  ;
    jsm = jsm + contrib;
    j = j - 1;
    if abs(jsm)>0
        fracerrj = abs(contrib/jsm);
    end
end     
vLOdd = jsm;

k = kmaxR;
ksm = 0;
fracerrk = 1;
while abs(ksm)>abstol & fracerrk > tol | k>kmaxR-5 
    contrib = RFt(k*T+Dt-t) .* RFxEven(k*X - xcen + disparity./2)  ;
    ksm = ksm + contrib;
    k = k - 1;
    if abs(ksm)>0
        fracerrk = abs(contrib/ksm);
    end
end     
vREven = ksm;

k = kmaxR;
ksm = 0;
fracerrk = 1;
while abs(ksm)>abstol & fracerrk > tol | k>kmaxR-5 
    contrib = RFt(k*T+Dt-t) .* RFxOdd(k*X - xcen + disparity./2)  ;
    ksm = ksm + contrib;
    k = k - 1;
    if abs(ksm)>0
        fracerrk = abs(contrib/ksm);
    end
end     
vROdd = ksm;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function r = RFxEven(x)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global twoSDxsq twopiSF
% Define RF function:
r = exp(-x.^2./twoSDxsq) .* cos(twopiSF.*x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function r = RFxOdd(x)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global twoSDxsq twopiSF
% Define RF function:
r = exp(-x.^2./twoSDxsq) .* sin(twopiSF.*x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function r = RFt(t)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global twoSDtsq SDt temporalag
% Define RF function:
r = (t<=0).*exp( - (t+temporalag).^2./twoSDtsq ); % Gaussian
%r = (t+temporalag<=0).*exp( (t+temporalag)./SDt ); % exponential