% Response of space-time RFs to random noise containing no motion or disparity signal.

% In this version, I separate the time-pixel used in simulating the RF, from the frame rate of the simulated monitor
% I also changed the sign convention to match Bruce's: Positive interocular delay = left sees a given image _after_ the right

% In this version, the motion ensors are monocular (RF in the left eye only).
% I also look at the effect of anti-correlated visual noise,
% simultaneously.

% In this version, I don't keep all the response, but average the
% correlation over many reps of ntpix runs.

% In this version, interocular delay is expressed in terms of tpix, not frames

% 12/Apr/05 In this version, I use Gaussian RFs, which match up with those
% used in the strobe Pulfrich effect.
% In this version, I specify slices through the Gaussian along the x and y
% axes,but specify the projection along the time axis.
% Also, although I don't seem to have commented on this before, each image
% is only presented for 1 frame, and the screen is assumed to be blank
% between frames (previously, I assumed each image lasted a full frame
% before being replaced by the next image, which was unrealistic).


clear all
close all
format compact
rand('state',2)
Pos=inline('x.*(x>0)');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Receptive fields
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
SDt = 10e-3; %s; projection of RF over x and y onto t axis is a Gaussian with SD SDt
SDshort = 0.02;% deg; t=0 slice through RF is a 2D Gaussian with SDs SDx and SDy.
SDlong = 0.06;% deg
vel = 10; % deg/s
ori = [0:90:270]*pi/180;
disparity = [-1:0.05:1]*0.5; % deg
temporalag = 50e-3;
DOWN=1; ori(DOWN)=0;
RIGHT=2;ori(RIGHT)=pi/2;
UP=3;ori(UP)=pi;
LEFT=4;ori(LEFT)=3*pi/2;

dxy = 0.0075;
mxx = 3*SDlong+0.5*max(abs(disparity));
mxy = 3*SDlong;
x = [-ceil(mxx/dxy)*dxy: dxy : ceil(mxx/dxy)*dxy];
y = [-mxy : dxy : mxy ];
nframesneedednow = 8; % This is the number of frames included in the receptive field trace
ntpixperframe = 10; % this is the number of time-pixels which equals one frame of the simulated monitor
nt = nframesneedednow * ntpixperframe; 
t = [-ceil(nt-1)/2:floor(nt)/2]/(nt-1)*2*5*SDt-temporalag;
lengthframe = min(diff(t))*ntpixperframe*1e3 % duration of 1 frame in ms
ntpix = 500; % the total number of t-pixels to simulate for in each run
kIOD = 10; % interocular delay in tpixels
nruns = 500; % the total number of runs to do, each lasting ntpix time-pixels.

tpix = abs(min(diff(t)));
xpix = abs(min(diff(x)));
nRFx = length(x);
nRFt = length(t);
nx = length(x);
ny = length(y);
ndisp = length(disparity);
nori = length(ori);
%ntf = length(TF);

[xp,yp,tp] = meshgrid(x,y,t); % size is (y,x,t)
% Set up direction sensors: 
om=atan(vel);
if vel~=0
    SD1sq = SDt/2/cos(om)^2 * (SDt + sqrt(SDt^2-SDshort^2*sin(2*om)^2));
    SD3sq = SDt/2/sin(om)^2 * (SDt - sqrt(SDt^2-SDshort^2*sin(2*om)^2));
else
    SD1sq = SDt^2;
    SD3sq = SDshort^2;
end    
% chekc:
cos(om)^2/SD3sq + sin(om)^2/SD1sq
1/SDshort^2
SD1sq*cos(om)^2 + SD3sq*sin(om)^2
SDt^2
if SD1sq<0 | SD3sq<0 | ~isreal(SD1sq) | ~isreal(SD3sq)
    disp('impossible values requested')
    SDt
    SDy*sin(2*om)
    stop
end
% theta=90: rightwards:
om = atan(-vel);
dir_RF{RIGHT} = exp( - (xp.*cos(om)-(tp+temporalag)*sin(om)).^2/2/SD3sq - yp.^2/2/SDlong^2 - (xp.*sin(om)+(tp+temporalag).*cos(om)).^2/2/SD1sq);
dir_RF{DOWN} = exp( - (yp.*cos(om)-(tp+temporalag)*sin(om)).^2/2/SD3sq - xp.^2/2/SDlong^2 - (yp.*sin(om)+(tp+temporalag).*cos(om)).^2/2/SD1sq);
om = atan(vel);
dir_RF{LEFT} = exp( - (xp.*cos(om)-(tp+temporalag)*sin(om)).^2/2/SD3sq - yp.^2/2/SDlong^2 - (xp.*sin(om)+(tp+temporalag).*cos(om)).^2/2/SD1sq);
dir_RF{UP} = exp( - (yp.*cos(om)-(tp+temporalag)*sin(om)).^2/2/SD3sq - xp.^2/2/SDlong^2 - (yp.*sin(om)+(tp+temporalag).*cos(om)).^2/2/SD1sq);
for jd=1:ndisp
    disp_RF{jd,DOWN} = exp( - yp.^2/2/SDshort^2 - (xp+disparity(jd)/2).^2/2/SDlong^2 - (tp+temporalag).^2/2/SDt^2);
    disp_RF{jd,UP} = disp_RF{jd,DOWN};
    disp_RF{jd,RIGHT} = exp( - (xp+disparity(jd)/2).^2/2/SDshort^2 - yp.^2/2/SDlong^2 - (tp+temporalag).^2/2/SDt^2);
    disp_RF{jd,LEFT} = disp_RF{jd,RIGHT};
end
SD1sq
SD3sq
clear xp yp tp

for jor=[UP DOWN]
	figure('pos',[ 176         299        1189         379])
	subplot(1,3,1)
	[mn,j]=min(abs(t-temporalag));
	imagesc(x,y,squeeze(disp_RF{find(disparity==0),jor}(:,:,j)));SquareAxis(gca);axis tight
	xlabel('x-position')
	ylabel('y-position')
	set(gca,'ydir','norm')
	subplot(1,3,2)
	imagesc(t,y,squeeze(disp_RF{find(disparity==0),jor}(:,fix(length(x)/2),:)));
	xlabel('time')
	ylabel('y-position')
	title('disparity sensor')
	set(gca,'ydir','norm')
	subplot(1,3,3)
	imagesc(t,y,squeeze(dir_RF{jor}(:,fix(length(x)/2),:)));
	if jor==RIGHT | jor==DOWN
		om = atan(-vel);
	else
		om=atan(vel);
	end
	hold on
	plot(-temporalag+[0 cos(om)],[0 sin(om)],'k')
	plot(-temporalag+[0 cos(om+pi/2)],[0 sin(om+pi/2)],'k--')
	xlabel('time')
	title('motion sensor')
	set(gca,'ydir','norm')
end
for jor=[LEFT RIGHT]
	figure('pos',[ 176         299        1189         379])
	subplot(1,3,1)
	[mn,j]=min(abs(t-temporalag));
	imagesc(x,y,squeeze(disp_RF{find(disparity==0),jor}(:,:,j)));SquareAxis(gca);axis tight
	xlabel('x-position')
	ylabel('y-position')
	set(gca,'ydir','norm')
	subplot(1,3,2)
	imagesc(t,x,squeeze(disp_RF{find(disparity==0),jor}(fix(length(y)/2),:,:)));
	xlabel('time')
	ylabel('x-position')
	title('disparity sensor')
	set(gca,'ydir','norm')
	subplot(1,3,3)
	imagesc(t,x,squeeze(dir_RF{jor}(fix(length(y)/2),:,:)));
	if jor==RIGHT | jor==DOWN
		om = atan(-vel);
	else
		om=atan(vel);
	end
	hold on
	plot(-temporalag+[0 cos(om)],[0 sin(om)],'k')
	plot(-temporalag+[0 cos(om+pi/2)],[0 sin(om+pi/2)],'k--')
	xlabel('time')
	title('motion sensor')
	set(gca,'ydir','norm')
end

figure
subplot(1,3,1)
[mn,jx]=min(abs(x));
[mn,jy]=min(abs(y));
plot(t,squeeze(sum(sum(dir_RF{RIGHT},1),2))/max(squeeze(sum(sum(dir_RF{RIGHT},1),2))))
hold on
plot(t,exp(-(t+temporalag).^2/2/SDt^2),'r:')
xlabel('t')
title('slice through x=0, y=0')
subplot(1,3,2)
[mn,jt]=min(abs(t+temporalag));
plot(x,squeeze(dir_RF{RIGHT}(jy,:,jt)))
hold on
plot(x,exp(-x.^2/2/SDshort^2),'r:')
xlabel('x')
title('slice through y=0, t=-lag')
subplot(1,3,3)
plot(y,squeeze(dir_RF{RIGHT}(:,jx,jt)))
hold on
plot(y,exp(-y.^2/2/SDlong^2),'r:')
xlabel('y')
title('slice through y=0, t=-lag')

figure('pos',[343   441   577   493]);
% positive IOD = right leads left

% Specify which disparities to look at:
jd_far = 17; % far
jd_near = 25; % near
jori_left = 2;
jori_right = 4;
% Initilaise
sumprod_farleft_corr = 0;
sumprod_farright_corr = 0;
sumprod_nearleft_corr = 0;
sumprod_nearright_corr = 0;
sumprod_farleft_anti = 0;
sumprod_farright_anti = 0;
sumprod_nearleft_anti = 0;
sumprod_nearright_anti = 0;
sum_corrn_corr = zeros(ndisp,nori);
sum_corrn_anti = zeros(ndisp,nori);
sum_corrn_corr_ODF = zeros(ndisp,nori);
sum_corrn_anti_ODF = zeros(ndisp,nori);
sum_corrn_corr_RCP = zeros(ndisp,nori);
sum_corrn_anti_RCP = zeros(ndisp,nori);

for jrun=1:nruns
    disp_response_corr = zeros(ndisp,nori,ntpix);
    disp_response_anti = disp_response_corr;
    disp_response_corr_RCP = disp_response_corr;
    disp_response_anti_RCP = disp_response_corr;
    disp_response_corr_ODF = disp_response_corr;
    disp_response_anti_ODF = disp_response_corr;
    dir_response = zeros(nori,ntpix);
    dir_response_RCP = dir_response;
    frames = (rand(ny,nx,ntpix./ntpixperframe+kIOD)>0.5)-0.5;
    jframe = [1:ntpixperframe:ntpix+kIOD]; % These are the indices at which there is an image presented on the screen (before any delaY)
    for jtpix=[1:ntpix]+nt
        fprintf('jrun=%d, jtpix=%d\n',jrun,jtpix)
        % Now work out which frames are displayed where in the RF
        thisjframeL = find(jframe >jtpix-nt & jframe<= jtpix);
        thisjframeR = find(jframe >jtpix+kIOD-nt & jframe<= jtpix+kIOD);
        
        for jor=1:nori
            v = sum(sum(sum(frames(:,:,thisjframeL).*dir_RF{jor}(:,:,1+jtpix-jframe(thisjframeL)) )));
            dir_response(jor,jtpix) =  v^2 ;
            dir_response_RCP(jor,jtpix) =  Pos(v)^2 ;
            % NB dir_response_ODF = dir_response_RCP
            for jd=1:ndisp
                jdR = find(disparity==-disparity(jd));
                vL=sum(sum(sum(frames(:,:,thisjframeL).*disp_RF{jd,jor}(:,:,1+jtpix-jframe(thisjframeL)) )));
                vR=sum(sum(sum(frames(:,:,thisjframeR).*disp_RF{jdR,jor}(:,:,1+jtpix+kIOD-jframe(thisjframeR)) )));
                
                disp_response_corr(jd,jor,jtpix) = ( vL + vR )^2 ;
                disp_response_anti(jd,jor,jtpix) = ( vL - vR )^2 ;
                disp_response_corr_ODF(jd,jor,jtpix) = (Pos( vL + vR ))^2 ;
                disp_response_anti_ODF(jd,jor,jtpix) = (Pos( vL - vR ))^2 ;
                disp_response_corr_RCP(jd,jor,jtpix) = ( Pos(vL) + Pos(vR) )^2 ;
                disp_response_anti_RCP(jd,jor,jtpix) = ( Pos(vL) + Pos(-vR) )^2 ;
            end % next disparity
        end % next orientation
    end % next time pixel
    for jd=1:ndisp
        for jor=1:nori
            corrn_corr(jd,jor) = Correlation(squeeze(disp_response_corr(jd,jor,:))',squeeze(dir_response(jor,:)));
            corrn_anti(jd,jor) = Correlation(squeeze(disp_response_anti(jd,jor,:))',squeeze(dir_response(jor,:)));
            corrn_corr_ODF(jd,jor) = Correlation(squeeze(disp_response_corr_ODF(jd,jor,:))',squeeze(dir_response_RCP(jor,:)));
            corrn_anti_ODF(jd,jor) = Correlation(squeeze(disp_response_anti_ODF(jd,jor,:))',squeeze(dir_response_RCP(jor,:)));   % NB dir_response_ODF = dir_response_RCP
            corrn_corr_RCP(jd,jor) = Correlation(squeeze(disp_response_corr_RCP(jd,jor,:))',squeeze(dir_response_RCP(jor,:)));
            corrn_anti_RCP(jd,jor) = Correlation(squeeze(disp_response_anti_RCP(jd,jor,:))',squeeze(dir_response_RCP(jor,:)));
        end
    end
    
    sum_corrn_corr = sum_corrn_corr + corrn_corr;
    sum_corrn_anti = sum_corrn_anti + corrn_anti;
    sum_corrn_corr_ODF = sum_corrn_corr_ODF + corrn_corr_ODF;
    sum_corrn_anti_ODF = sum_corrn_anti_ODF + corrn_anti_ODF;
    sum_corrn_corr_RCP = sum_corrn_corr_RCP + corrn_corr_RCP;
    sum_corrn_anti_RCP = sum_corrn_anti_RCP + corrn_anti_RCP;
    
    subplot(1,2,1)
    plot(disparity,sum_corrn_corr./jrun)
    title('correlated noise')
    drawnow
    hold on
    plot(disparity,sum_corrn_corr_ODF./jrun,'--')
    plot(disparity,sum_corrn_corr_RCP./jrun,':')
    hold off
    subplot(1,2,2)
    plot(disparity,sum_corrn_anti./jrun)
    title('anti-correlated noise')
    hold on
    plot(disparity,sum_corrn_anti_ODF./jrun,'--')
    plot(disparity,sum_corrn_anti_RCP./jrun,':')
    set(gcf,'name',num2str(jrun))
    hold off
    drawnow
    
    save RandomNoise16_ntpixperframe=10_IOD=10tpix_vel=10_SD1=0.1_SD3=0.002_SDt=10ms.mat
    
end


