function SimulationForFigures789
clear all
close all hidden
format compact
format long
global twoSDxsq twoSDtsq twopiSF SDt temporalag twopiTF
global ntime ndisparity tol
% This is based on the maths I have laid out in longversion.doc.

% This program outputs the response of a population of model neurons tuned
% to different stimulus positions and disparities, at different times
% throughout a strobe Pulfrich stimulus

% Define RF function
SDx = 0.1; % deg
SDt = 10e-3; %s
SF = 2; %cycles per deg
temporalag = 50e-3;
TF = 20;

% Define stimulus properties
TToverSDt = [2:6]; % temporal period of strobe stimulus, in s
vel = 3.6; % speed of strobe stimulus, in deg/s
dttoverT = [0:0.05:0.5];

% Accuracy parameters. This is what "acc5" in the save file refers to
ndisparity = 301;
nxcen = 61;
ntime = 151;
tol = 5e-6;

twopiSF = 2*pi*SF;
twopiTF = 2*pi*TF;
twoSDxsq = 2*SDx^2;
twoSDtsq = 2*SDt^2;

col={'r' 'b' [0 0.5 0] 'm' 'g' 'c'};
monocterm = zeros(length(dttoverT),ntime,nxcen,ndisparity);
dispmodterm = zeros(length(dttoverT),ntime,nxcen,ndisparity);
figure
for jT=1:length(TToverSDt)
    T=TToverSDt(jT)*SDt;
    X = vel .* T; % spatial period of strobe stimulus, in deg
    time = [0:ntime-1]*T/ntime;
    for jdt=1:length(dttoverT)
        dtoverT=dttoverT(jdt);
        Dt = dtoverT*T;
        % Calculate the maximum disparity we can encounter
        dispmax = 4 * SDx + abs(vel) * (4*SDt + abs(Dt));
        dispmin = -dispmax;
        disparity = [ dispmin : (dispmax-dispmin)/(ndisparity-1) : dispmax  ] ; 
        
        vL = zeros(nxcen,ndisparity);
        vR = vL;
        for jt=1:ntime    
            xmin = - 4 * SDx - abs(X);
            xmax = 4 * SDx + abs(X);
            xcen = [ xmin : (xmax-xmin)/(nxcen-1) : xmax]  + vel*time(jt) - vel*temporalag;
            for jx=1:nxcen
                for jd=1:ndisparity
                    [vL(jx,jd),vR(jx,jd)] = Convolutions(time(jt),X,T,Dt,xcen(jx),disparity(jd));
                end
            end
            imagesc(vL)
            drawnow
			monocterm(jdt,jt,:,:) = vL.^2+vR.^2 ;      
            dispmodterm(jdt,jt,:,:) = 2.*vL.*vR ;       
        end
        disparitydistrib = squeeze(sum(sum(dispmodterm(jdt,:,:,:),2),3))';
        percdisp(jT,jdt) = sum(disparitydistrib.*disparity)./sum(disparitydistrib);
        save(sprintf('Fullresults_simple_gauss_Tontau=%1d_acc5.mat',TToverSDt(jT)),'SDx','SDt','SF','TF','temporalag','vel','dttoverT','ndisparity','nxcen','ntime','tol','dispmodterm','T','X','monocterm');
    end % next delay dt
    plot(dttoverT,percdisp(jT,:)/X,'col',col{jT},'linestyle','-','linew',2);
    hold on
    plot(dttoverT,DisparityAveraging_RFconv(dttoverT*T,T,'Gaussian',SDt),'col',col{jT},'linestyle',':','linew',2);
    xlabel('dt/T')
    ylabel('dx/X')
    drawnow
end % next strobe period T


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [vL,vR] = ContinuousCase(t,X,T,Dt,xcen,disparity)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vel = X/T;
vLEven = quad(@RFL,-100*X,100*X,1e-7,[],xcen,disparity,vel,t);
vREven = quad(@RFR,-100*X,100*X,1e-7,[],xcen,disparity,vel,t,Dt);


function z = RFL(x,xcen,Dxpref,vel,t)
z = RFx(x-xcen-Dxpref/2).*RFt(x/vel - t);
function z = RFR(x,xcen,Dxpref,vel,t,Dt)
z = RFx(x-xcen+Dxpref/2).*RFt(x/vel + Dt - t);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [vL,vR] = Convolutions(t,X,T,Dt,xcen,disparity)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global tol

abstol=1e-30;

jmaxL = floor( t/T);
kmaxR = floor( (t-Dt)/T);

j = jmaxL;
jsm = 0;
fracerrj = 1;
while abs(jsm)>abstol & fracerrj > tol | j>jmaxL-15 
    % To avoid making it spend ages converging accurately on a tiny value,
    % I force it to loop at least 15 times, but then stop if the answer is
    % still no larger than abstol.
    % NB! This can screw things up if the temporal kernel is such that uou
    % need to you back more than 15 periods to get a response
    contrib = RFt(j*T-t) .* RFx(j*X - xcen - disparity./2)  ;
    jsm = jsm + contrib;
    j = j - 1;
    if abs(jsm)>0
        fracerrj = abs(contrib/jsm);
    end
end     
vL = jsm;

k = kmaxR;
ksm = 0;
fracerrk = 1;
while abs(ksm)>abstol & fracerrk > tol | k>kmaxR-15 
    contrib = RFt(k*T+Dt-t) .* RFx(k*X - xcen + disparity./2)  ;
    ksm = ksm + contrib;
    k = k - 1;
    if abs(ksm)>0
        fracerrk = abs(contrib/ksm);
    end
end     
vR = ksm;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function r = RFx(x)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global twoSDxsq twopiSF
% Define RF function:
r = exp(-x.^2./twoSDxsq) .* cos(twopiSF.*x);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function r = RFt(t)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global twoSDtsq SDt temporalag twopiTF
% Define RF function:
r = (t<=0).*exp( - (t+temporalag).^2./twoSDtsq ); % Gaussian
%r = (t+temporalag<=0).*exp( (t+temporalag)./SDt ); % exponential
%r = (t<=0).*exp(-(t+temporalag).^2./twoSDtsq) .* cos(twopiTF.*(t+temporalag)); % test1

